package org.hansv.db;

import au.com.bytecode.opencsv.CSVReader;
import java.io.*;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;

/**
 * Manage data from CSV file
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
public class Csv implements Serializable{
	private String locPath;
	private char[] charColSep;
	private  char[] charStrSep;
	private String locCharset;
	private boolean locHeader;
	private ErrorLog errlog;
	private Messages msg;
	
	public Csv(){

	}
	
	/**
	 * Set local variables
	 * @param path Path CSV file
	 * @param sepCol Character of column separateor
	 * @param sepStr Character of string separator
	 * @param charset Caharacter set
	 * @param header Flag if csv file has header.
	 */
	public void setUp(String path, String sepCol, String sepStr, String charset, boolean header){
		locPath = path;
		locHeader = header;  
		charColSep = getColumnDelimiter(sepCol);
		charStrSep = sepStr.toCharArray(); 
		locCharset = getCharset(charset);
	}
	
	/**
	 * Setter injector for class ErrorLog
	 * @param er Class ErrorLog
	 * @see org.hansv.utils.ErrorLog
	 */
	public void setErrorLog(ErrorLog er){
		errlog = er;
	}
	
	/**
	 * Setter injector for class Messages
	* @param m Class Messages
	* @see org.hansv.utils.Messages
	*/
	public void setMessages(Messages m){
		msg  = m;
	}
	
	
    
	/**
	 * Return correct column delimiter
	 * @param separator delimiter of columns in table
	 * @return correct column delimiter
	 */
	private char[] getColumnDelimiter(String separator){
		switch(separator){
			case "mezera":
				separator = " ";
				break;
			case "TAB":
				separator = "\t";
				break;
		}
		//prevedeni Stringu na pole char. CSV reader vyzaduje argumenty typu char
		return separator.toCharArray();
	}
	
	/**
	 * Return correct strinct of charset
	 * @param charset Name of charset
	 * @return String
	 */
	private String getCharset(String charset){
		String charset2 = null;
		switch(charset){
			case "Windows 1250":
				charset2 = "cp1250";
				break;
			case "ISO 8859-2":
				charset2 ="ISO8859_2";
				break;
			case "UTF 8":
				charset2="UTF8";
				break;
			case "cp852":
				charset2 = "cp852";
				break;
		}
		return charset2;
	}


	/**
	 * Get data of csv file
	 * @return enveloped data in 2D array
	 */
	public Object[][] readData(){
		Object[][] data;
		int i = 0;
		//zamena viceznakovych vyjadreni oddelovace sloupcu na jejich jednoznakove reprezentace
		
		try{
			CSVReader reader;
			reader = getReader();
			
			String [] nextLine;
			//zjisteni pocet radek v CSV souboru pro dimenzovavi jednoho rozmeru pole s daty
			while ((nextLine = reader.readNext()) != null) {
				i++;
			}
			reader.close();
			
			//vytvoreni pole s daty
			CSVReader readerData;
			readerData = getReader();
			Object[][] data1 = new Object[i][];
			int j = 0;
			while ((nextLine = readerData.readNext()) != null) {
				data1[j] = nextLine;
				j++;
			}
			
			//kopirovani pole do promenne s platnosti pro celou proceduru
			data=data1;
		}catch(Exception e){
			data = new Object[2][2];
			data[0][0] = ExceptionUtils.getStackTrace(e);
			String descr = "Cteni CSV souboru, DbTxt.readData";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
		return data;
	}
	
	/**
	 * Get data of part CSV file
	 * @return Object[][]
	 */
	public Object[][] readDataSample(){
		Object[][] data;
		int i = 0;
		
		try{
			CSVReader reader;
			reader = getReader();
			
			String [] nextLine;
			//zjisteni pocet radek v CSV souboru pro dimenzovavi jednoho rozmeru pole s daty
			while ((nextLine = reader.readNext()) != null) {
				i++;
			}
			reader.close();
			
			//vytvoreni pole s daty
			if(i>50){
				i=50;
			}
			CSVReader readerData;
			readerData = getReader();
			Object[][] data1 = new Object[i][];
			int j = 0;
			while (j<i) {
				nextLine = readerData.readNext();
				data1[j] = nextLine;
				j++;
			}
			//kopirovani pole do promenne s platnosti pro celou proceduru
			data=data1;
		}catch(Exception e){
			data = new Object[1][1];
			data[0][0] = "Error";
			String descr = "Cteni CSV souboru, Csv.readDataSample";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
		return data;
	}
	
	/**
	 * Return CSVReader
	 * @return CSVReader
	 */
	private CSVReader getReader(){
		CSVReader readerData = null;
		try{
			if (locHeader == true){
				readerData = new CSVReader(new InputStreamReader(new FileInputStream(locPath),locCharset),charColSep[0],charStrSep[0],1);
			}else{
				readerData = new CSVReader(new InputStreamReader(new FileInputStream(locPath),locCharset),charColSep[0],charStrSep[0]);
			}
		}catch(IOException e){
			String descr = "Cteni CSV souboru, Csv.getReader";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
		return readerData;
		
	}
	
	/**
	 * Return names of columns
	 * @return String[]
	 */
	public String[] getColumns(){
		String[] data;
		int i = 0;
		try{
			CSVReader reader = new CSVReader(new FileReader(locPath));
			String [] nextLine;
			//zjisteni pocet radek v CSV souboru pro dimenzovavi jednoho rozmeru pole s daty
			while ((nextLine = reader.readNext()) != null) {
				i++;
			}
			reader.close();
			
			//vytvoreni pole s daty
			if(i>0){
			
			
				CSVReader readerData = new CSVReader(new InputStreamReader(new FileInputStream(locPath),locCharset),charColSep[0],charStrSep[0]);
				String[] data1 = new String[1];
				int j = 0;
				while (j<1) {
					nextLine = readerData.readNext();
					data1 = nextLine;
					j++;
				}
				
				//kopirovani pole do promenne s platnosti pro celou proceduru
				if(locHeader == true){
					data=data1;
				}else{
					data = new String[data1.length];
					for(int k=0; data1.length > k; k++){
						data[k] = "Sloupec " + (k+1);                     
					}
				}
			}else{
				data = new String[1];
				data[0] =  "No data";
			}
		}catch(IOException e){
			data = new String[1];
			data[0] =  "No data";
			String descr = "Chyba čtení CSV souboru, Csv.readDataSample";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
		return data;
	}
}
