package org.hansv.model;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.db.H2db;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 */
public class DecompPlaceModel {
	protected ErrorLog errlog;
	protected Messages msg;
	protected H2db db;
	private InputImportModel iim;
	
	/**
	 * Setter injector for class ErrorLog
	 * @param er Class ErrorLog
	 * @see org.hansv.utils.ErrorLog
	 */
	public void setErrorLog(ErrorLog er){
		errlog = er;
	}

	/**
	 * Setter injector for class Messages
	 * @param m Class Messages
	 * @see org.hansv.utils.Messages
	 */
	public void setMessages(Messages m){
		msg  = m;
	}

	/**
	 * Setter injector for class H2db
	 * @param h2 Class H2db
	 * @see org.hansv.db.H2db
	 */
	public void setH2db(H2db h2){
		db  = h2;
	}
	
	/**
	 * Setter injector for class InputImportModel
	 * @param iim Class CoordsourceModel
	 * @see org.hansv.model.InputImportModel
	 */
	public void setInputImportModel(InputImportModel iim){
		this.iim = iim;
	}
	
	/**
	 * Get namedplaces and localities from table input
	 * @param place named place
	 * @return
	 */
	public ResultSet getData(String place){
		Connection cnn = db.getConn();
		ResultSet rs = null;
		try{
			if(place != null){
				PreparedStatement pstmt = cnn.prepareStatement("SELECT DISTINCT NAMEDPLACE AS \"Staré jméno\", NAMEDPLACE AS \"Nové jméno\", LOCALITY AS \"Lokalita\" FROM input WHERE NAMEDPLACE =?",ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
				pstmt.setString(1, place);
				pstmt.execute();
				rs = pstmt.executeQuery();
			}
			return rs;
		}catch(SQLException e){
			e.printStackTrace();
			String descr = "Nezdařilo se načtení dat z tabulky";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
			return null;
		}
	}
	
	/**
	 * Update table INPUT, NAMEDPLACE in transaction
	 * @param data
	 */
	public void save(String[][] data){
		Connection cnn = db.getConn();
		PreparedStatement pstmtInput;
		PreparedStatement pstmtDel;
		PreparedStatement pstmtNP;
		try{
			cnn.setAutoCommit(false);
			pstmtInput = cnn.prepareStatement("UPDATE INPUT SET NAMEDPLACE = ? WHERE NAMEDPLACE = ? AND LOCALITY = ?");
			for(int i=0; i<data.length; i++){				
				pstmtInput.setString(1, data[i][0]);
				pstmtInput.setString(2, data[i][1]);
				pstmtInput.setString(3, data[i][2]);
				pstmtInput.executeUpdate();
			}
			pstmtDel = cnn.prepareStatement("DELETE FROM NAMEDPLACE WHERE ORIG = ?");
			pstmtDel.setString(1, data[0][1]);
			pstmtDel.executeUpdate();
			pstmtNP = cnn.prepareStatement("INSERT INTO NAMEDPLACE(ORIG) VALUES(?)");
			ArrayList newPlaces = getNewPlaces(data);
			for(int i = 0; i< newPlaces.size(); i++){
				pstmtNP.setString(1, newPlaces.get(i).toString());
				pstmtNP.executeUpdate();
			}
			cnn.commit();
		}catch(SQLException e){
			e.printStackTrace();
			String descr = "Nezdařilo se uložení dat";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	/**
	 * 
	 * Get ArrayList of named places
	 * @param data
	 * @return 
	 */
	private ArrayList getNewPlaces(String[][] data){
		ArrayList<String> place = new ArrayList<>(0);
		for(int i = 0;i<data.length; i++){
			if(isInList(place, data[i][0]) == false){
				place.add(data[i][0]);
			}
		}
		return place;
	}
	
	/**
	 * Check if the plase is in ArrayList
	 * @param places ArrayList of named places
	 * @param place checked named place
	 * @return
	 */
	private boolean isInList(ArrayList places, String place){
		boolean res = false;
		if(places != null){
			for(int i = 0;i<places.size();i++){
				if(places.get(i).equals(place)){
					res = true;
				}
			}
		}
		return res;
	}
	
	/**
	 * Get unique coords of locality
	 * @param place Named place
	 * @param locality Description of locality
	 * @return
	 */
	public ResultSet getCoordsLoc(String place, String locality){
		Connection cnn = db.getConn();
		ResultSet rs = null;
		try{
			if(place != null){
				PreparedStatement pstmt = cnn.prepareStatement("SELECT DISTINCT  LATITUDE, LONGITUDE FROM input WHERE NAMEDPLACE =? AND LOCALITY = ?",ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
				pstmt.setString(1, place);
				pstmt.setString(2, locality);
				pstmt.execute();
				rs = pstmt.executeQuery();
			}
			return rs;
		}catch(SQLException e){
			e.printStackTrace();
			String descr = "Nezdařilo se načtení dat z tabulky";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
			return null;
		}
	}
}
