package org.hansv.model;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javafx.concurrent.Service;
import javafx.concurrent.Task;
import javafx.scene.control.TreeItem;
import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.db.H2db;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;


/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
public class InputImportModel extends Service<String>{

	private ErrorLog errlog;
	private Messages msg;
	private H2db db;
	private List<String> columnsList;
	private Object[][] data;
	private List treeList;
	public AtomicBoolean shouldThrow = new AtomicBoolean(false);


	public InputImportModel(){
	
	}
	
	/**
	 * Setter injector for class ErrorLog
	 * @param er Class ErrorLog
	 * @see org.hansv.utils.ErrorLog
	 */
	public void setErrorLog(ErrorLog er){
		errlog = er;
	}
	
	/**
	 * Setter injector for class Messages
	 * @param m Class Messages
	 * @see org.hansv.utils.Messages
	 */
	public void setMessages(Messages m){
		msg  = m;
	}
	
	/**
	 * Setter injector for class H2db
	 * @param h2 Class H2db
	 * @see org.hansv.db.H2db
	 */
	public void setH2db(H2db h2){
		db  = h2;
	}
	
	/**
	 * Create table "input" for store input data
	 */
	public void createTable(){
		String sqlDrop = "DROP TABLE IF EXISTS input";
		String sqlTable = "CREATE TABLE input("
							+ "id_input bigint auto_increment primary key,"    
							+ "taxon varchar(1000),"
							+ "taxonid varchar(100),"
							+ "originaltaxon varchar(1000),"
							+ "abbrev varchar(1000),"
							+ "area  varchar(1000),"
							+ "phytochorion varchar(1000),"
							+ "namedplace varchar(1000),"
							+ "country varchar(1000),"
							+ "netmaparea varchar(1000),"
							+ "locality varchar(1000),"
							+ "localityid varchar(100),"
							+ "latitudeorig  varchar(1000),"
							+ "longitudeorig varchar(1000),"
							+ "latitude  double,"
							+ "longitude double,"
							+ "distance double,"
							+ "coordaccuracy  varchar(1000),"
							+ "coordsource varchar(100),"
							+ "altitude  varchar(1000),"
							+ "biotope  varchar(1000),"
							+ "abund  varchar(1000),"
							+ "localitynote  varchar(1000),"
							+ "occurencenote varchar(1000),"
							+ "schedanote varchar(1000),"
							+ "date1 VARCHAR(10),"
							+ "date2 VARCHAR(10),"
							+ "date1orig varchar(1000),"
							+ "date2orig varchar(1000),"
							+ "collectors  varchar(1000),"
							+ "herbarium varchar(1000),"
							+ "source  varchar(1000),"
							+ "sourceid  varchar(100),"
							+ "project varchar(1000),"
							+ "projectid varchar(100),"
							+ "negative varchar (1000) DEFAULT '0',"
							+ "schedacount varchar (3) DEFAULT '1',"
							+ "determinated varchar (1000),"
							+ "guid varchar (100),"
							+ "collectorsid  varchar(1000),"
							+ "revised varchar (1000),"
							+ "importid INT)";
		Connection cnn = db.getConn();
		try{
			Statement stmt = cnn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,ResultSet.CONCUR_READ_ONLY);
			stmt.execute(sqlDrop);
			stmt.execute(sqlTable);
			cnn.close();
		}catch(SQLException e){
			String descr = "Nezdařilo se smazání a opětovné vytvoření tabulky input";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
		db.closeConn();
	}
	
	/**
	 * Set necessary variable for import
	 * @param columnsList List with columns names of imported CSV file
	 * @param data Array with data from javaFX table
	 * @param treeList Tree with mapped columns
	 */
	public void setParams(List<String> columnsList, Object[][] data, List treeList){
		this.data = data;
		this.columnsList = columnsList;
		this.treeList = treeList;
	}

	
	
	/**
	 * Return map with id from list for csv columns and name from database columns
	 * @param columnsList List with columns names of imported CSV file
	 * @param treeList Tree with mapped columns
	 * @return Map
	 */
	private Map getMap(List<String> columnsList, List treeList){
		TreeItem treeItemDatabase;
		TreeItem treeItemInput;
		Map<String, Integer> columnsMap = new HashMap<>();
		for(Object o:treeList){
			treeItemDatabase = (TreeItem)o;
			if(treeItemDatabase.getChildren().isEmpty() == false){
				for(Object item: treeItemDatabase.getChildren()){
					treeItemInput = (TreeItem)item;
					columnsMap.put(treeItemInput.getValue().toString().substring(4), getId(columnsList,treeItemInput.getValue().toString().substring(4)));
				}
			}
		}
		return columnsMap;
	}
	
	/**
	 * Get identifier of column name from list
	 * @param columnList List with columns names of imported CSV file
	 * @param columnName Column name
	 * @return integer
	 */
	private Integer getId(List<String> columnList,String columnName){
		Integer id = 0;
		Integer i = 0;
		for(String name: columnList){
			if(name.equals(columnName)){
				id = i;
			}
			i++;
		}
		return id;
	}
	
	/**
	 * Get all data from table input
	 * @return ResultSet
	 */
	public ResultSet getData(){
		String sql = "SELECT id_input, taxon, originaltaxon AS `Originální jméno taxonu`, abbrev AS `Přesnost určení`, area AS `Oblast`, phytochorion AS `Fytochorion`, namedplace AS `Sídlo`,country as `Stát` ,netmaparea AS `Kvadrant`, locality AS `Lokalita`, latitudeorig AS `Originální zeměpisná šířka`, latitude AS `Zeměpisná šířka`, longitudeorig AS `Originální zeměpisná délka`, longitude AS `Zeměpisná délka`, distance AS `Vzdálenost lokality od sídla`, coordaccuracy AS `Přesnost souřadnic`, coordsource AS `Zdroj souřadnic`,altitude AS `Nadmořská výška`, biotope AS `Biotop`, abund AS `Početnost`, localitynote AS `Poznámka k lokalitě`, occurencenote AS `Poznámka k nálezu`, schedanote AS  `Poznámka na schedě`, schedacount AS `Počet sched/listů`, date1orig AS `Originální datum 1`, date1 AS `Datum 1`, date2orig AS `Originální datum 2`, date2 AS `Datum 2`, collectors AS `Nálezci`, herbarium AS `Herbář`, source AS `Zdroj`, project AS `Projekt`,negative AS `Negativní nález`, determinated  AS `Determinoval`, revised AS `Revidoval` FROM input;";
		Connection cnn = db.getConn();
		try{
			Statement stmt = cnn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
			ResultSet rs = stmt.executeQuery(sql);
			return rs;
		}catch(SQLException e){
			String descr = "Nezdařilo se získání dat z tabulky input";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
			return null;
		}
	}
	
	/**
	 * Get not imported occurences from table input
	 * @return ResultSet
	 * @throws SQLException
	 */
	public ResultSet getNotImportedOccurence() throws SQLException{
		String sql = "SELECT * FROM input WHERE guid IS NULL;";
		Connection cnn = db.getConn();
		Statement stmt = cnn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
		ResultSet rs = stmt.executeQuery(sql);
		return rs;
	}
	
	/**
	 * Get not imported collectors from table input
	 * @return ResultSet
	 * @throws SQLException
	 */
	public ResultSet getNotImportedCollectors() throws SQLException{
		String sql = "SELECT * FROM input WHERE collectors_saved = false;";
		Connection cnn = db.getConn();
		Statement stmt = cnn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
		ResultSet rs = stmt.executeQuery(sql);
		return rs;
	}
	
	/**
	 * Get data from table input for saving coordinates and date
	 * @return ResultSet
	 */
	public ResultSet getUpdatableData(){
		String sql = "SELECT id_input,  latitudeorig , latitude, longitudeorig , longitude, date1 , date1orig ,  date2orig , date2  FROM input;";
		Connection cnn = db.getConn();
		try{
			Statement stmt = cnn.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE);
			ResultSet rs = stmt.executeQuery(sql);
			return rs;
		}catch(SQLException e){
			String descr = "Nezdařilo se získání dat z tabulky input";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
			return null;
		}
	}
	
	/**
	 * Update value in specific column
	 * @param id Identifier of record. In database table is as int but in TableView is as String
	 * @param newValue New value
	 * @param columnName Column name in form from TableView
	 */
	public void saveValue(String id, String newValue, String columnName){
		Connection cnn = db.getConn();
		String sql = "UPDATE INPUT SET " + getColumnName(columnName) + " = ? WHERE ID_INPUT = ?";
		try{
			PreparedStatement pstmt = cnn.prepareStatement(sql);
			pstmt.setString(1, newValue);
			pstmt.setString(2, id);
			pstmt.execute();
			cnn.close();
		}catch(SQLException e){
			String descr = "Nezdařilo se uložit novou hodnotu do tabulky input";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	public String getColumnName(String columnLabel){
		String name = null;
		HashMap<String,String> map = new HashMap<>();
		map.put("TAXON", "taxon");
		map.put("ORIGINÁLNÍ JMÉNO TAXONU", "originaltaxon");
		map.put("PŘESNOST URČENÍ", "abbrev");
		map.put("OBLAST", "area");
		map.put("FYTOCHORION","phytochorion");
		map.put("SÍDLO", "namedplace");
		map.put("STÁT", "country");
		map.put("KVADRANT", "netmaparea");
		map.put("LOKALITA","locality");
		map.put("ZEMĚPISNÁ ŠÍŘKA", "latitude");
		map.put("ZEMĚPISNÁ DÉLKA","longitude");
		map.put("ORIGINÁLNÍ ZEMĚPISNÁ ŠÍŘKA", "latitudeorig");
		map.put("ORIGINÁLNÍ ZEMĚPISNÁ DÉLKA","longitudeorig");
		map.put("PŘESNOST SOUŘADNIC", "coordaccuracy");
		map.put("ZDROJ SOUŘADNIC","coordsource");
		map.put("NADMOŘSKÁ VÝŠKA","altitude");
		map.put("BIOTOP", "biotope");
		map.put("POČETNOST", "abund");
		map.put("POZNÁMKA K LOKALITĚ","localitynote");
		map.put("POZNÁMKA K NÁLEZU","occurencenote");
		map.put("DATUM 1","date1");
		map.put("DATUM 2","date2");
		map.put("ORIGINÁLNÍ DATUM 1","date1orig");
		map.put("ORIGINÁLNÍ DATUM 2","date2orig");
		map.put("NÁLEZCI","collectors");
		map.put("HERBÁŘ", "herbarium");
		map.put("ZDROJ","source");
		map.put("PROJEKT", "project");
		map.put("NEGATIVNÍ NÁLEZ","negative");
		map.put("DETERMINOVAL","determinated");
		map.put("REVIDOVAL", "revised");
		map.put("POZNÁMKA NA SCHEDĚ", "schedanote");
		map.put("POČET SCHED/LISTŮ", "schedacount");
		name = map.get(columnLabel);
		return name;
	}
	
	
	//@Override
	protected Task<String> createTask() {
		return new Task<String>(){
			//@Override
			protected String call() throws Exception{
				Connection cnn = db.getConn();
				String sql = "INSERT INTO input (taxon, originaltaxon, abbrev, area, phytochorion, namedplace,country ,netmaparea, locality, latitudeorig, longitudeorig, coordaccuracy, coordsource, altitude, biotope, abund, localitynote, occurencenote, date1orig, date2orig, collectors, herbarium, source, project, negative, determinated, revised, importid, schedacount, schedanote) VALUES "
							+ "(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?);";
				Map colsMap = getMap(columnsList, treeList);
				PreparedStatement pstmt = cnn.prepareStatement(sql);
				int i = 0;
				int total = data.length;
				String negative = "0";
				Random rnd = new Random();
				Integer importid  = Math.abs(rnd.nextInt(10000000));
				for(Object[] o:data){
					if(StringUtils.isBlank(getValue(o, treeList, 23, colsMap))== false){
						negative = getValue(o, treeList, 23, colsMap);
					}
					pstmt.setString(1,getValue(o, treeList, 0, colsMap));
					pstmt.setString(2,getValue(o, treeList, 1, colsMap));
					pstmt.setString(3,getValue(o, treeList, 2, colsMap));
					pstmt.setString(4,getValue(o, treeList, 3, colsMap));
					pstmt.setString(5,getValue(o, treeList, 4, colsMap));
					pstmt.setString(6,getValue(o, treeList, 5, colsMap));
					pstmt.setString(7,getValue(o, treeList, 6, colsMap));
					pstmt.setString(8,getValue(o, treeList, 7, colsMap));
					pstmt.setString(9,getValue(o, treeList, 8, colsMap));
					pstmt.setString(10,getValue(o, treeList, 9, colsMap));
					pstmt.setString(11,getValue(o, treeList, 10, colsMap));
					pstmt.setString(12,getValue(o, treeList, 11, colsMap));
					pstmt.setString(13,getValue(o, treeList, 12, colsMap));
					pstmt.setString(14,getValue(o, treeList, 13, colsMap));
					pstmt.setString(15,getValue(o, treeList, 14, colsMap));
					pstmt.setString(16,getValue(o, treeList, 15, colsMap));
					pstmt.setString(17,getValue(o, treeList, 16, colsMap));
					pstmt.setString(18,getValue(o, treeList, 17, colsMap));
					pstmt.setString(19,getValue(o, treeList, 18, colsMap));
					pstmt.setString(19,getValue(o, treeList, 18, colsMap));
					pstmt.setString(20,getValue(o, treeList, 19, colsMap));
					pstmt.setString(21,getValue(o, treeList, 20, colsMap));
					pstmt.setString(22,getValue(o, treeList, 21, colsMap));
					pstmt.setString(23,getValue(o, treeList, 22, colsMap));
					pstmt.setString(24,negative);
					pstmt.setString(25,getValue(o, treeList, 24, colsMap));
					pstmt.setString(26,getValue(o, treeList, 25, colsMap));
					pstmt.setString(27,getValue(o, treeList, 26, colsMap));
					pstmt.setInt(28, importid);
					pstmt.setString(29,getValue(o, treeList, 27, colsMap));
					pstmt.setString(30,getValue(o, treeList, 28, colsMap));
					pstmt.execute();
					i++;
					updateProgress(i, total);
				}
				db.closeConn();
				return "Completed";
			}

			/**
			 * Get value of record from table for csv data
			 * @param data Record of csv data
			 * @param treeList Mapped column
			 * @param index Index of column
			 * @param colsMap Map with columns names and mapped column
			 * @return String
			 */
			private String getValue(Object[] data, List treeList, int index, Map<String, Integer> colsMap) {
				String value = "";
				List<TreeItem> colsInput;
				Matcher m;
				TreeItem ti = (TreeItem)treeList.get(index);
				if(ti.getChildren().isEmpty() == false){
					colsInput = ti.getChildren();
					Pattern p = Pattern.compile("^'.+'$");
					for(TreeItem s:colsInput){
						if(value.length() > 0){
							value += "; ";
						}
						m = p.matcher(s.getValue().toString().trim());
						if(m.matches()){
							value += s.getValue().toString().trim().substring(1, s.getValue().toString().trim().length()-1);
						}else{
							value += data[colsMap.get(s.getValue().toString().substring(4))];
						}
					}
				}
				if(value.length() == 0){
					value = null;
				}else{
					value = value.trim();
				}
				return value;
			}
		};
	}
	
	/**
	 * Set flag into table input, if collector was imported
	 * @param id Identifier of record
	 * @throws SQLException
	 */
	public void setIsCollectorsImported(int id) throws SQLException{
		Connection cnn = db.getConn();
		String sql = "UPDATE input SET collectors_saved = true WHERE id_input = ?";
		PreparedStatement pstmt = cnn.prepareStatement(sql);
		pstmt.setInt(1, id);
		pstmt.execute();
		cnn.close();
	}
	
	/**
	 * Set value into column taxonid in table input by condition
	 * @param value Value in condition
	 * @param valueid New value in column taxonid
	 * @see #setColumn(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	public void setTaxonId(String value, String valueid){
		setColumn("input", "taxonid", valueid, "taxon", value);
	}

/**
	 * Set value into column country in table input by condition
	 * @param idinput id_input
	 * @param countrycode Country code
	 * @see #setColumn(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	public void setCountryCode(int idinput, String countrycode){
		setColumn("input", "country", countrycode , "id_input", idinput);
	}
	
	/**
	 * Set value into column sourceid in table input by condition
	 * @param value Value in condition
	 * @param valueid New value in column sourceid
	 * @see #setColumn(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	public void setSourceId(String value, String valueid){
		setColumn("input", "sourceid", valueid, "source", value);
	}
	
	/**
	 * Set value into column sourceid in table input by condition
	 * @param value Value in condition
	 * @param valueid New value in column sourceid
	 * @see #setColumn(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	public void setProjectId(String value, String valueid){
		setColumn("input", "projectid", valueid, "project", value);
	}
	
	/**
	 * Set value into column collectorsid in table input by condition
	 * @param value Value in condition
	 * @param valueid New value in column taxonid
	 * @see #setColumn(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	public void setCollectorsId(String value, String valueid){
		setColumn("input", "collectorsid", valueid, "collectors", value);
	}
	
	
	
	/**
	 * Set value in specific column by condition
	 * @param table name of table
	 * @param column1 column with changed values
	 * @param value1 new value
	 * @param column2 column in condition
	 * @param value2 value in condition
	 */
	private void setColumn(String table, String column1, String value1, String column2 , String value2){
		Connection cnn = db.getConn();
		String sql = "UPDATE " + table +" SET " + column1 + " = ? WHERE "+ column2 + " = ?";
		try{
			PreparedStatement pstmt = cnn.prepareStatement(sql);
			pstmt.setString(1, value1);
			pstmt.setString(2, value2);
			pstmt.execute();
			cnn.close();
		}catch(SQLException e){
			String descr = "Nezdařilo se aktualizovat sloupec " + column1 +" v tabulce " + table;
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}

	private void setColumn(String table, String column1, String value1, String column2 , int value2){
		Connection cnn = db.getConn();
		String sql = "UPDATE " + table +" SET " + column1 + " = ? WHERE "+ column2 + " = ?";
		try{
			PreparedStatement pstmt = cnn.prepareStatement(sql);
			pstmt.setString(1, value1);
			pstmt.setInt(2, value2);
			pstmt.execute();
			cnn.close();
		}catch(SQLException e){
			String descr = "Nezdařilo se aktualizovat sloupec " + column1 +" v tabulce " + table;
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	/**
	 * Set locality id in table localities
	 * @param guid identifier of locality from server
	 * @param id identifier of record
	 * @throws SQLException
	 */
	public void setOccurenceId(String guid, int id) throws SQLException{
		Connection cnn = db.getConn();
		String sql = "UPDATE input SET guid = ? WHERE id_input = ?";
		PreparedStatement pstmt = cnn.prepareStatement(sql);
		pstmt.setString(1, guid);
		pstmt.setInt(2, id);
		pstmt.execute();
		cnn.close();
	}
	
	public void setLocalityIdOcc(String idlocality, String locality, double longitude, double latitude, String place, String area, String phytochorion, String netmaparea, String coordaccuracy, String coordsource, String altitude, String localitynote) throws SQLException{
		Connection cnn = db.getConn();
		String sql = "UPDATE input SET localityid = '" + StringEscapeUtils.escapeSql(idlocality) +"' WHERE locality " + getValue(locality) + " AND namedplace " + getValue(place) +" AND longitude = " + longitude + " AND latitude = " + latitude + " AND area " + getValue(area) + " AND phytochorion " + getValue(phytochorion) + " AND netmaparea " + getValue(netmaparea) + " AND coordaccuracy " + getValue(coordaccuracy) + " AND coordsource " + getValue(coordsource) + " AND altitude  " + getValue(altitude) + " AND  localitynote  " + getValue(localitynote);
		Statement stmt = cnn.createStatement();
		stmt.execute(sql);
		cnn.close();
	}
	
	public void saveDistances(String place, String longitude, String latitude){
		String sql = "UPDATE input SET distance = ROUND(111.045* DEGREES(ACOS(COS(RADIANS(?)) * COS(RADIANS(latitude)) * COS(RADIANS(?) - RADIANS(longitude)) + SIN(RADIANS(?)) * SIN(RADIANS(latitude)))),3) WHERE namedplace = ?;";
		Connection cnn = db.getConn();
		try{
			PreparedStatement pstmt = cnn.prepareStatement(sql);
			pstmt.setDouble(1, Double.parseDouble(latitude));
			pstmt.setDouble(2, Double.parseDouble(longitude));
			pstmt.setDouble(3, Double.parseDouble(latitude));
			pstmt.setString(4, place);
			pstmt.execute();
			cnn.close();
		}catch(SQLException e){
			String descr = "Nezdařilo se uložit vzdálenost mezi sídlem a lokalitou";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	public ResultSet getDistanceAnalyse(String place){
		String sql = "SELECT MAX(distance) AS max, MIN(distance) AS min, COUNT(distance) as count FROM input WHERE namedplace = ?";
		Connection cnn = db.getConn();
		try{
			PreparedStatement pstmt = cnn.prepareStatement(sql);
			pstmt.setString(1, place);
			ResultSet rs = pstmt.executeQuery();
			return rs;
		}catch(SQLException e){
			String descr = "Nezdařilo se získání dat z tabulky input";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
			return null;
		}
	}
	
	public ResultSet getDistanceAnalyse(){
		String sql = "SELECT MAX(distance) AS max, MIN(distance) AS min, COUNT(distance) as count FROM input WHERE distance IS NOT NULL";
		Connection cnn = db.getConn();
		try{
			Statement stmt = cnn.createStatement();
			ResultSet rs = stmt.executeQuery(sql);
			return rs;
		}catch(SQLException e){
			String descr = "Nezdařilo se získání dat z tabulky input";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
			return null;
		}
	}
	
	/**
	 * Get count localities of named place in specific intervals
	 * @param place
	 * @return 
	 */
	public ResultSet getCountSpecificDistance(String place){
		String sql = "SELECT COUNT(distance) as count FROM input WHERE namedplace = ?";
		Connection cnn = db.getConn();
		try{
			PreparedStatement pstmt = cnn.prepareStatement(sql);
			pstmt.setString(1, place);
			ResultSet rs = pstmt.executeQuery();
			return rs;
		}catch(SQLException e){
			String descr = "Nezdařilo se získání dat z tabulky input";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
			return null;
		}
	}
	
	
	public ResultSet getTaxon(){
		Connection cnn = db.getConn();
		ResultSet rs = null;
		try{
			Statement stmt = cnn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
			rs = stmt.executeQuery("SELECT id_input, taxon FROM input");
		}catch(SQLException e){
			String descr = "Nezdařilo se načtení dat z tabulky input";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
			return null;
		}
		return rs;
	}
	
	/**
	 * Save new value for taxon
	 * @param taxon
	 * @param id 
	 * @param cf
	 */
	public void saveTaxon(String taxon, int id, String cf){
		saveValue(String.valueOf(id), taxon, "TAXON");
		if(cf != null){
			saveValue(String.valueOf(id), cf, "PŘESNOST URČENÍ");
		}
	}
	
	/**
	 * Save null distance of specific named place
	 * @param place
	 */
	public void saveNullDistance(String place){
		Connection cnn = db.getConn();
		String sql = "UPDATE INPUT SET DISTANCE = NULL WHERE  NAMEDPLACE = ?";
		try{
			PreparedStatement pstmt = cnn.prepareStatement(sql);
			pstmt.setString(1, place);
			pstmt.execute();
			cnn.close();
		}catch(SQLException e){
			String descr = "Nezdařilo se vynulovat vzdálenost lokalit od sídla " + place;
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	/**
	 * Save null distance of specific named place
	 */
	public void saveNullDistance(){
		Connection cnn = db.getConn();
		String sql = "UPDATE INPUT SET DISTANCE = NULL";
		try{
			Statement stmt = cnn.createStatement();
			stmt.execute(sql);
			cnn.close();
		}catch(SQLException e){
			String descr = "Nezdařilo se vynulovat vzdálenost lokalit od sídel";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	/**
	 * Get occurences for specific locality
	 * @param locality
	 * @param longitude
	 * @param latitude
	 * @param place
	 * @param area
	 * @param phytochorion
	 * @param netmaparea
	 * @param coordaccuracy
	 * @param coordsource
	 * @param altitude
	 * @param localitynote
	 * @param country
	 * @return OccurenceItem[]
	 * @throws java.sql.SQLException
	 */
	public OccurenceItem[] getOccurences4locality(String locality, double longitude, double latitude, String place, String area, String phytochorion, String netmaparea, String coordaccuracy, String coordsource, String altitude, String localitynote, String country) throws SQLException{
		String sql = "SELECT taxonid, taxon, originaltaxon, abbrev, biotope, abund, occurencenote, schedanote, schedacount, date1, date2, collectorsid, herbarium, sourceid, projectid, negative, determinated, revised FROM input WHERE locality " + getValue(locality) + " AND namedplace " + getValue(place) +" AND longitude = " + longitude + " AND latitude = " + latitude + " AND area " + getValue(area) + " AND phytochorion " + getValue(phytochorion) + " AND netmaparea " + getValue(netmaparea) + " AND coordaccuracy " + getValue(coordaccuracy) + " AND coordsource " + getValue(coordsource) + " AND altitude  " + getValue(altitude) + " AND localitynote " + getValue(localitynote);
		Connection cnn = db.getConn();
		OccurenceItem[] oi = null;
		Statement stmt =  cnn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
		ResultSet rs = stmt.executeQuery(sql);
		rs.last();
		int count = rs.getRow();
		rs.beforeFirst();
		oi = new OccurenceItem[count];
		int i = 0;
		while(rs.next()){
			oi[i] = new OccurenceItem(null, rs.getString("taxonid"), rs.getString("abbrev"), rs.getString("originaltaxon"),rs.getString("taxon"), rs.getString("date1"), rs.getString("date2"), rs.getString("sourceid"), rs.getString("projectid"), rs.getString("occurencenote"), rs.getString("schedanote"), rs.getString("schedacount"), rs.getString("herbarium"), rs.getString("biotope"), rs.getString("abund"), rs.getString("determinated"), rs.getString("revised"), "3", rs.getString("negative"), rs.getString("collectorsid"), null, null);
			i++;
		}
		cnn.close();
		return oi;
	}
	
	private String getValue(String val){
		if(val == null){
			return "IS NULL";
		}else{
			return " = '" + StringEscapeUtils.escapeSql(val) + "'";
		}
	}
	
	public ResultSet getDuplicity(){
		String sql = "SELECT count(*) AS pocet, taxon, latitudeorig ,  longitudeorig , locality, localitynote , occurencenote ,  date1orig , date2orig ,  collectors, schedanote  FROM input GROUP BY taxon, latitudeorig ,  longitudeorig , locality, localitynote , occurencenote ,  date1orig , date2orig ,  collectors, schedanote HAVING  count(*) > 1";
		Connection cnn = db.getConn();
		try{
			Statement stmt = cnn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
			ResultSet rs = stmt.executeQuery(sql);
			return rs;
		}catch(SQLException e){
			String descr = "Nezdařilo se získání duplicitních dat z tabulky input";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
			return null;
		}
	}
	
	/**
	 * Get count of all data in table input
	 * @return 
	 */
	public int getCountData() throws SQLException{
		Connection cnn = db.getConn();
		Statement stmt = cnn.createStatement();
		ResultSet rs = stmt.executeQuery("SELECT count(*) FROM input");
		rs.first();
		return rs.getInt(1);
	}
	
	public void deleteDuplicity(){
		ResultSet rs = getDuplicity();
		
		try{
			while(rs.next()){
				deleteRow(rs.getString("taxon"), rs.getString("latitudeorig"), rs.getString("longitudeorig"), rs.getString("locality"), rs.getString("localitynote"), rs.getString("occurencenote"), rs.getString("date1orig"), rs.getString("date2orig"), rs.getString("collectors"));
			}
		}catch(SQLException e){
			String descr = "Chyba při mazání duplicit";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	private String getWhereparam(String param){
		if(param == null){
			return "IS NULL";
		}else{
			return "= '" + StringEscapeUtils.escapeSql(param) + "'";
		}
	}
	
	private void deleteRow(String taxon, String latitudeorig, String longitudeorig, String locality, String localitynote, String occurencenote, String date1orig, String date2orig, String collectors){
		Connection cnn = db.getConn();
		ArrayList<Integer> ids = new ArrayList<>();
		try{
			//System.out.println("SELECT id_input FROM input WHERE taxon "+ getWhereparam(taxon) + " AND latitudeorig " + getWhereparam(latitudeorig) +" AND  longitudeorig " + getWhereparam(longitudeorig) + " AND locality " + getWhereparam(locality) + " AND localitynote " + getWhereparam(localitynote) + " AND occurencenote " + getWhereparam(occurencenote) + " AND  date1orig " + getWhereparam(date1orig) + " AND date2orig " + getWhereparam(date2orig) + " AND  collectors " + getWhereparam(collectors));
			Statement stmt = cnn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
			ResultSet rs1 = stmt.executeQuery("SELECT id_input FROM input WHERE taxon "+ getWhereparam(taxon) + " AND latitudeorig " + getWhereparam(latitudeorig) +" AND  longitudeorig " + getWhereparam(longitudeorig) + " AND locality " + getWhereparam(locality) + " AND localitynote " + getWhereparam(localitynote) + " AND occurencenote " + getWhereparam(occurencenote) + " AND  date1orig " + getWhereparam(date1orig) + " AND date2orig " + getWhereparam(date2orig) + " AND  collectors " + getWhereparam(collectors));
			rs1.last();
			int count = rs1.getRow();
			rs1.beforeFirst();
			for(int i = 1; i<count; i++){
				rs1.next();
				ids.add(rs1.getInt("id_input"));
			}
			rs1.close();
			for(int ii = 0; ii<ids.size(); ii++){
				stmt.execute("DELETE FROM input WHERE id_input = " + ids.get(ii));
			}
		}catch(SQLException e){
			String descr = "Chyba při mazání duplicit";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	public void deleteLocalityId(){
		Connection cnn = db.getConn();
		String sql = "UPDATE input SET localityid = null";
		try{
		Statement stmt = cnn.createStatement();
		stmt.execute(sql);
		cnn.close();
		}catch(Exception e){
			e.printStackTrace();
		}
	}
	
	/**
	 * Get some coordinate of one locality by place  
	 * @param place
	 * @return 
	 */
	public Double[] getCoordslocalityByPlace(String place){
		Connection cnn = db.getConn();
		ResultSet rs = null;
		Double[] coord = new Double[2];
		try{
			PreparedStatement pstmt = cnn.prepareStatement("SELECT longitude, latitude FROM input WHERE namedplace = ? limit 1");
			pstmt.setString(1, place);
			pstmt.execute();
			rs = pstmt.executeQuery();
			rs.first();
			coord[0] = rs.getDouble("latitude");
			coord[1] = rs.getDouble("longitude");
		}catch(SQLException e){
			e.printStackTrace();
			String descr = "Nezdařilo se načtení souřadnic";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
			return null;
		}
		return coord;
	}
}

