package org.hansv.model;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import javafx.collections.ObservableList;
import javafx.concurrent.Service;
import javafx.concurrent.Task;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.db.H2db;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;

/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 * @version 1.0
 */
public abstract class MapTableModel extends Service<String> implements IMapTable{
	protected ErrorLog errlog;
	protected Messages msg;
	protected H2db db;
	protected RestModel rm;
	protected String table;
	protected ObservableList<ObservableList> data;
	
	
	/**
	 * Setter injector for class ErrorLog
	 * @param er Class ErrorLog
	 * @see org.hansv.utils.ErrorLog
	 */
	public void setErrorLog(ErrorLog er){
		errlog = er;
	}

	/**
	 * Setter injector for class Messages
	 * @param m Class Messages
	 * @see org.hansv.utils.Messages
	 */
	public void setMessages(Messages m){
		msg  = m;
	}

	/**
	 * Setter injector for class H2db
	 * @param h2 Class H2db
	 * @see org.hansv.db.H2db
	 */
	public void setH2db(H2db h2){
		db  = h2;
	}
	
	/**
	 * Create table for mapping
	 */
	public void createTable(){
		Connection cnn = db.getConn();
		try{
			Statement stmt = cnn.createStatement();
			stmt.execute("DROP TABLE IF EXISTS  "+ table + ";");
			String sql = "CREATE TABLE "+ table + "("
				+ "id int auto_increment primary key,"
				+ "orig varchar(500),"
				+ "mapids varchar(255),"
				+ "mapname varchar(500))";
			stmt.execute(sql);
			db.closeConn();
		}catch(SQLException e){
			String descr = "Nezdařilo se smazání a opětovné vytvoření tabulky "+ table;
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	
	
	
	
	/**
	 * Save data into table
	 * @param data String array with saved data
	 */
	public void save(String[] data){
		Connection cnn = db.getConn();
		try {
			PreparedStatement pstmt = cnn.prepareStatement("UPDATE "+ table + " SET mapname = ?, mapids = ? WHERE orig = ?;");
			pstmt.setString(1, data[1]);
			pstmt.setString(2, data[2]);
			pstmt.setString(3, data[0]);
			pstmt.execute();
		}catch(SQLException e){
			String descr = "Nezdařilo se uložení";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	/**
	 * Set necessary variable for mapping
	 * @param data1 Data from JavaFX Table
	 * @param rm1 Class RestModel
	 * @see org.hansv.model.RestModel
	 */
    public void setParams(ObservableList<ObservableList> data1, RestModel rm1){
        data = data1;
		rm = rm1;
    }
	
	@Override
	protected Task<String> createTask() {
		return new Task<String>(){
			@Override
			protected String call() throws Exception {
				ArrayList restvalues;
				String[] restdata;
				String[] values = new String[3];
				if(data != null){
					for(int i = 0; i< data.size();i++){
						ObservableList row = data.get(i);
						restvalues = getRest(row.get(1).toString());
						if(restvalues.size() == 1){
							restdata = (String[])restvalues.get(0);
							values[0] = row.get(1).toString();
							values[1] = restdata[1];
							values[2] = restdata[0];
							row.set(2, restdata[1]);
							row.set(3, restdata[0]);
							data.set(i, row);
							save(values);
						}
						updateProgress(i + 1, data.size());
					}
				}
				return "Completed";
			}
		};
	}
	
	public int getUnmapped() throws SQLException{
		Connection cnn = db.getConn();
		int res = -1;
		Statement stmt = cnn.createStatement();
		String sql = "SELECT COUNT(*) AS COUNT FROM " + table + " WHERE mapids IS NULL;";
		ResultSet rs = stmt.executeQuery(sql);
		rs.first();
		res = rs.getInt("count");
		return res;
	}
	
	public boolean isAnalyzed() throws SQLException{
		Connection cnn = db.getConn();
		int res;
		Statement stmt = cnn.createStatement();
		String sql = "SELECT COUNT(*) AS COUNT FROM " + table + ";";
		ResultSet rs = stmt.executeQuery(sql);
		rs.first();
		res = rs.getInt("count");
		if(res == 0){
			return false;
		}else{
			return true;
		}

	}
	
	protected abstract ArrayList getRest(String sample);
	
	public String findId(String value) throws SQLException{
		Connection cnn = db.getConn();
		String id = null;
		if(value != null){
			PreparedStatement pstmt = cnn.prepareStatement("SELECT mapids AS id FROM " + table + " WHERE orig =?");
			pstmt.setString(1, value);
			pstmt.execute();
			ResultSet rs = pstmt.executeQuery();
			if(rs != null){
				rs.first();
				id = rs.getString("id");
			}
		}
		return id;
	}
	
}
