package org.hansv.model;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import javafx.collections.ObservableList;
import javafx.concurrent.Task;
import org.apache.commons.lang.exception.ExceptionUtils;


/**
 * @author Vladimír Hans, BSD licence http://opensource.org/licenses/bsd-license.php
 */
public class NamedPlaceModel extends MapTableModel{
	
	private InputImportModel iim;
	
	/**
	 * Setter injector for class InputImportModel
	 * @param iim Class CoordsourceModel
	 * @see org.hansv.model.InputImportModel
	 */
	public void setInputImportModel(InputImportModel iim){
		this.iim = iim;
	}
	
	public NamedPlaceModel(){
		table = "namedplace";
	}
	
	
	/**
	 * Create table for mapping
	 */
	@Override
	public void createTable(){
		Connection cnn = db.getConn();
		try{
			Statement stmt = cnn.createStatement();
			stmt.execute("DROP TABLE IF EXISTS  "+ table + ";");
			String sql = "CREATE TABLE "+ table + "("
				+ "id int auto_increment primary key,"
				+ "orig varchar(500),"
				+ "mapids varchar(255),"
				+ "mapname varchar(500),"
				+ "longitude double,"
				+ "latitude double,"
				+ "maxdistance double)";
			stmt.execute(sql);
			db.closeConn();
		}catch(SQLException e){
			String descr = "Nezdařilo se smazání a opětovné vytvoření tabulky "+ table;
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	/**
	 * Get all data from table project
	 * @return ResultSet
	 */
	@Override
	public ResultSet getData(){
		Connection cnn = db.getConn();
		try{
			Statement stmt = cnn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
			ResultSet rs = stmt.executeQuery("SELECT id, orig AS `Nejbližší sídlo`, mapname AS `Mapované sídlo`, mapids AS `Kód mapovaného sídla`, latitude AS `Zeměpisná šířka`, longitude AS `Zeměpisná délka`, maxdistance AS `Maximální vzdálenost` FROM namedplace WHERE orig IS NOT NULL");
			return rs;
		}catch(SQLException e){
			String descr = "Nezdařilo se načtení dat z tabulky " + table;
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
			return null;
		}
	}
	
	/**
	 * Import unique named places into table namedplaces
	 */
	@Override
	public void importData(){
		Connection cnn = db.getConn();
		try{
			Statement stmt = cnn.createStatement();
			String sql = "INSERT INTO " + table + " (orig) SELECT DISTINCT namedplace FROM input WHERE namedplace IS NOT NULL";
			stmt.execute(sql);
			db.closeConn();
		}catch(SQLException e){
			String descr = "Nezdařil se import jedinečných sídel do tabulky "+ table;
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
		
	@Override
	protected ArrayList getRest(String sample){
		return rm.getNamedPlaces2(sample);
	}
	
	/**
	 * Save data into table
	 * @param data String array with saved data
	 */
	@Override
	public void save(String[] data){
		Connection cnn = db.getConn();
		try {
			PreparedStatement pstmt = cnn.prepareStatement("UPDATE "+ table + " SET mapname = ?, mapids = ?, longitude = ?, latitude = ?, maxdistance = ? WHERE orig = ?;");
			pstmt.setString(1, data[1]);
			pstmt.setString(2, data[2]);
			pstmt.setString(6, data[0]);
			if(data[3] != null){
				pstmt.setDouble(3, Double.parseDouble(data[3]));
			}else{
				pstmt.setNull(3,java.sql.Types.FLOAT);
			}
			if(data[4] != null){
				pstmt.setDouble(4, Double.parseDouble(data[4]));
			}else{
				pstmt.setNull(4, java.sql.Types.FLOAT);
			}
			if(data[5] != null){
				pstmt.setDouble(5, Double.parseDouble(data[5]));
			}else{
				pstmt.setNull(5, java.sql.Types.FLOAT);
			}
			pstmt.execute();
		}catch(SQLException e){
			String descr = "Nezdařilo se uložení";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	@Override
	protected Task<String> createTask() {
		return new Task<String>(){
			@Override
			protected String call() throws Exception {
				ArrayList restvalues;
				String[] restdata;
				String[] values = new String[6];
				if(data != null){
					for(int i = 0; i< data.size();i++){
						ObservableList row = data.get(i);
						restvalues = getRest(row.get(1).toString());
						if(restvalues.size() == 1){
							restdata = (String[])restvalues.get(0);
							values[0] = row.get(1).toString();
							values[1] = restdata[1];
							values[2] = restdata[0];
							values[3] = restdata[2];
							values[4] = restdata[3];
							row.set(2, restdata[1]);
							row.set(3, restdata[0]);
							row.set(4,restdata[3]);
							row.set(5, restdata[2]);
							iim.saveDistances(values[0],values[3],values[4]);
							ResultSet distance = iim.getDistanceAnalyse(values[0]);
							distance.first();
							row.set(6, Double.toString(distance.getDouble("max")));
							values[5] = Double.toString(distance.getDouble("max"));
							data.set(i, row);
							save(values);
							
						}
						updateProgress(i + 1, data.size());
					}
				}
				return "Completed";
			}
		};
	}
}
