package org.hansv.vratickaimport3;

import com.google.inject.Inject;
import java.io.IOException;
import java.net.URL;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TablePosition;
import javafx.scene.control.TableView;
import javafx.scene.control.cell.TextFieldTableCell;
import javafx.scene.layout.Pane;
import javafx.scene.web.WebView;
import javafx.stage.Stage;
import javafx.stage.StageStyle;
import javafx.util.Callback;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.hansv.model.DecompPlaceModel;
import org.hansv.utils.ErrorLog;
import org.hansv.utils.Messages;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * FXML Controller class
 *
 * @author vladimir.hans
 */
public class DecompPlaceController implements Initializable {

	protected Map<String,Integer> columnsNames = new HashMap<>();
	private DecompPlaceModel model;
	private Messages msg;
	private ObservableList<ObservableList> data;
	private ErrorLog errlog;

	
	@FXML
	private TableView table_data;
	@FXML
	private Button decompplace_save;
	@FXML
	private Button decompplace_map;
	@FXML
	private WebView www_analyze;
	private NamedPlaceController npc;
	
	/**
	 * Initializes the controller class.
	 */
	@Override
	public void initialize(URL url, ResourceBundle rb) {
		//System.out.println("initialize");
		ApplicationContext context = new ClassPathXmlApplicationContext(new String[]{"config/Beans.xml"});
		errlog = (ErrorLog)context.getBean("errorLog");
		msg = (Messages)context.getBean("messages");
		model = (DecompPlaceModel)context.getBean("decompPlaceModel");
		decompplace_save.setDisable(false);
	}
	
	@FXML
	public void handleUpdate(){
		for(ObservableList row:data){
			if(row.get(1).toString() == null){
				msg.showWarn("Nejsou doplněna všechna nová jména", "Nutno doplnit všechny nová jména i v případě, že jsou stejná jako stará");
				return;
			}
		}
		String[][] place = new String[data.size()][3];
		int i = 0;
		for(ObservableList row:data){
			place[i][0] = row.get(1).toString();
			place[i][1] = row.get(0).toString();
			place[i][2] = row.get(2).toString();
			i++;
		}
		model.save(place);
		decompplace_save.setDisable(true);
		npc.handleReload();
	}
	
	@FXML
	public void handleMap(){
		ObservableList selectedCells = table_data.getSelectionModel().getSelectedCells();
		if(selectedCells.isEmpty()){
			msg.showWarn("Nemáte vybranou lokalitu, kterou chcete zobrazit na mapě", "Vyberte ji v tabulce");
			return;
		}
		TablePosition tablePosition = (TablePosition) selectedCells.get(0);
		ObservableList row = data.get(tablePosition.getRow());
		Double[] coord = getCoord(row.get(1).toString(), row.get(2).toString());
		showMap(coord[0], coord[1]);
	}
	
	/**
	 * Show www browser with coords of localitzy on mapy.cz
	 * @param longitude
	 * @param latitude 
	 */
	private void showMap(Double longitude, Double latitude){
		try {
			FXMLLoader loader = new FXMLLoader(getClass().getResource("/fxml/WWWBrowser.fxml"));
			Stage stage = new Stage(StageStyle.DECORATED);
			stage.setScene(new Scene((Pane) loader.load()));
			WWWBrowserController controller = loader.<WWWBrowserController>getController();
			controller.loadMap(longitude, latitude);
			stage.show();
			
		} catch (IOException e) {
			String descr = "Nezdařilo se otevřít okno s vestavěným WWW prohlížečem";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	/**
	 * Initialize data in table
	 * @param place named place
	 */
	public void initData(String place){
		setTableColumns(place);
		loadData(place);
	}
	
	public void setNamedPlaceController(NamedPlaceController npcc){
		npc = npcc;
	}
	
	/**
	 * Load data in table
	 * @param place 
	 */
	private void loadData(String place){
		ResultSet rs = model.getData(place);
		int recordCount;
		try{
			ResultSetMetaData rsmd = rs.getMetaData();
			rs.last();
			recordCount = rs.getRow();
			rs.first();
			data = FXCollections.observableArrayList();
			for(int i = 0; i < recordCount; i++){
				ObservableList<String> row = FXCollections.observableArrayList();
				for(int j = 1;j <= rsmd.getColumnCount();j++ ){
					if(StringUtils.isBlank(rs.getString(j))){
						row.add("");
					}else{
						row.add(rs.getString(j));
					}
				}
				data.add(row);
				rs.next();
			}
			table_data.setItems(data);
		}catch(SQLException e){
			e.printStackTrace();
			String descr = "Nezdařilo se získání dat z tabulky";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	/**
	 * Get coord of localities
	 * @param place
	 * @param locality
	 * @return Double[0] = longitude, Double[1] = latitude
	 */
	private Double[] getCoord(String place, String locality){
		Double[] coords = new Double[2];
		ResultSet rs = model.getCoordsLoc(place, locality);
		try{
			rs.last();
			int count = rs.getRow();
			rs.beforeFirst();
			rs.first();
			coords[0] = rs.getDouble("LONGITUDE");
			coords[1] = rs.getDouble("LATITUDE");
			if(count > 1){
				msg.showInfo("K lokalitě se vztahuje " + count + " jedinečných souřadnic. Ns mapě bude zobrazena jen jedna z nich", "Více jedinečných souřadnic k lokalitě");
			}
		}catch(SQLException e){
			String descr = "Chyba v v seznamu souřadnic lokality";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
		return coords;
	}
	
	/**
	 * Set columns in table
	 * @param place named place
	 */
	private void setTableColumns(String place){
		ResultSet rs = model.getData(place);
		table_data.setEditable(true);
		table_data.getColumns().clear();
		List<String> columns = new ArrayList<>();
		try{
			ResultSetMetaData rsmd = rs.getMetaData();
			int ii = 0;
			while(ii< rsmd.getColumnCount()){
				ii++;
				columns.add(rsmd.getColumnLabel(ii));
			};
			int i = 0;
			for(String columnName : columns) {
				final int j = i;
				TableColumn col = new TableColumn(columnName);
				col.setCellValueFactory(new Callback<TableColumn.CellDataFeatures<ObservableList,String>,ObservableValue<String>>(){
					public ObservableValue<String> call(TableColumn.CellDataFeatures<ObservableList, String> param) {
						return new SimpleStringProperty(param.getValue().get(j).toString());
					}
				});
				if(columnName.equals("Staré jméno")){
					col.setEditable(false);
				}
				if(columnName.equals("Nové jméno")){
					col.setEditable(true);
				}
				if(columnName.equals("Lokalita")){
					col.setEditable(false);
				}else{
					col.setCellFactory(TextFieldTableCell.forTableColumn());
					col.setOnEditCommit(new EventHandler<TableColumn.CellEditEvent>(){
						@Override
						public void handle(TableColumn.CellEditEvent event) {
							saveChangeValue(event);
						}
					});
				}
				
				columnsNames.put(columnName,i);
				table_data.getColumns().addAll(col);
				i++;
			}
			
		}catch(SQLException e){
			String descr = "Nezdařilo se nastavit sloupce pro tabulku sídel";
			errlog.logErr(ExceptionUtils.getStackTrace(e), descr);
			String[] errs = ExceptionUtils.getRootCauseStackTrace(e);
			msg.showErr(errs[0] + errs[1], descr);
		}
	}
	
	private void saveChangeValue(TableColumn.CellEditEvent event){
		ObservableList row = data.get(event.getTablePosition().getRow());
		row.set(columnsNames.get(event.getTableColumn().getText()), event.getNewValue());
		data.set(event.getTablePosition().getRow(), row);
	}
	
}
